#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QTimer>
#include <QDesktopServices>
#include <QUrl>
#include <QDir>
#include <QFileDialog>
#include <QMessageBox>
#include <QCloseEvent>
#include <QSettings>
#include <QCoreApplication>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    QTimer *timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(OneSec()));
    timer->start(1000); // start executing OneSec() every  second
    connect(ui->pushButton_browse, SIGNAL(clicked()), this, SLOT(Browse()));
    connect(ui->checkBox_alarmEnabled, SIGNAL(toggled(bool)), this, SLOT(AlarmSet(bool)));
    connect(ui->actionExit, SIGNAL(triggered()), this, SLOT(close()));
    connect(ui->actionAbout, SIGNAL(triggered()), this, SLOT(About()));
    connect(ui->actionHelp, SIGNAL(triggered()), this, SLOT(Help()));
    readSettings();
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::readSettings()
{
    QSettings settings(QSettings::IniFormat,QSettings::UserScope,"krubow","alarmclock"); // constructor, force INI file storage
    ui->timeEdit_alarmtime->setTime(settings.value("alarmtime").toTime());
    ui->lineEdit_action->setText(settings.value("action").toString());
}

void MainWindow::AlarmSet(bool state)
{
    if (state) { // if alarm was turned on
        QDateTime now=QDateTime::currentDateTime();
        alarmDate=now.date();
        if (ui->timeEdit_alarmtime->time() < QDateTime::currentDateTime().time()) { // alarm time is already past
            alarmDate=alarmDate.addDays(1); // set alarm for next day (tomorrow)
            // note: there can be problems if alarm time is changed while alarm is enabled
        }
    }
}

// Note: Triggering the alarm is trickier that it appears at first.
// Looking for current time equalls alarm time seems logical,
// but doesn't work if computer is busy at the alarm time and OneSec doesn't get called until later.
// By the time OneSec is called the alarm time is past, and equality is missed.
// On the other hand, checking for current time >= alarm time doesn;t work either
// since the alarm would be triggered immediately if the alarm time is earlier than current time.
// In this case the alarm should not be triggered until tomorrow.
// So I need alarmDate as well as alarm time, and trigger the alarm only if currentdatetime>=alarmdatetime
void MainWindow::OneSec(void)
{
    QDateTime curDateTime=QDateTime::currentDateTime();
    QDateTime alarmDateTime(alarmDate,ui->timeEdit_alarmtime->time());
    ui->label_time->setText(curDateTime.toString("hh:mm:ss AP"));
    if (ui->checkBox_alarmEnabled->isChecked() && curDateTime>=alarmDateTime) {
        ui->checkBox_alarmEnabled->setChecked(false);
        // launch the action (this is like double-clicking the selected file, typically to play a sound file)
        QDesktopServices::openUrl(QUrl("file:///"+ui->lineEdit_action->text(),QUrl::TolerantMode));
    }
}

void MainWindow::Browse(void)
{
    QString filename;
    static QString mydir=QDir::homePath(); // last visited directory

    filename=QFileDialog::getOpenFileName(this,"File to launch",mydir,"All files (*.*)");
    if (filename!="") {
        QDir path(filename);
        mydir=path.absolutePath(); // remember last visited directory
        ui->lineEdit_action->setText(filename);
    }
}

void MainWindow::About()
{
    QMessageBox::about(this,"AlarmClock","<center><h1>AlarmClock 1.00</h1>by Keith Rubow<br><a href=\"www.krubow.com\">www.krubow.com</a></center>");
}

void MainWindow::Help()
{
    QDesktopServices::openUrl(QUrl("file:///"+QCoreApplication::applicationDirPath()+"/AlarmClock.html"));
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    QSettings settings(QSettings::IniFormat,QSettings::UserScope,"krubow","alarmclock"); // constructor, force INI file storage
    settings.setValue("alarmtime",ui->timeEdit_alarmtime->time()); // save alarm time
    settings.setValue("action",ui->lineEdit_action->text()); // save the action file name
    event->accept();
}
